/*
 *  tcpjunk - TCP protocols testing tool
 *  Contact: oferkv@gmail.com
 *
 *  tcpjunk is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2, or (at your option) any
 *  later version.
 *
 *  tcpjunk is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include "tcpjunk.h"

void junk(struct tcpjunk_data instance_data)
  {
  char send_buf[SESSION_MAX_LEN];
  int send_buf_len = 0;
  int rc;
  int cl_socket;
  int r_timeout;
  struct sockaddr_in sa, ba;

  /* socket stuff */
  memset(&sa, 0, sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons(instance_data.server_port);
  memcpy(&sa.sin_addr.s_addr, instance_data.server_addr,
      instance_data.server_addr_len);

  /* socket binding stuff */
  struct tcpjunk_data bind_data;
  bind_data.debug = 1;
  memset(&ba, 0, sizeof(ba));
  ba.sin_family = AF_INET;
  char bind_if_name[32];
  char new_src_ip[32];

  /* other stuff */
  int buf_id;
  int loop_counter = instance_data.loop;
  int keepalive_counter = instance_data.keepalive;

  /* SSL Stuff */
  int err;
  SSL_CTX *ctx = NULL;
  SSL *ssl = NULL;
  SSL_METHOD *meth;

  /* Prepare SSL Stuff if needed */
  if (instance_data.use_ssl == 1)
    {
    SSL_library_init();
    SSL_load_error_strings();

    /* Create SSL_CTX structure */
    meth = SSLv3_method();
    ctx = SSL_CTX_new(meth);
    if ((ctx) == NULL)
      {
      tj_error(&instance_data, "Cannot create an SSL_CTX structure", 0);
      exit(EXIT_FAILURE);
      }
    /* SSL structure */
    ssl = SSL_new(ctx);
    if ((ssl) == NULL)
      {
      tj_error(&instance_data, "Cannot create an SSL structure", 0);
      exit(EXIT_FAILURE);
      }
    }

  /**
   ***  main loop
   **/
  while (instance_data.loop == 0 ? 1 : loop_counter--)
    {
    /* load random session data file */
    if (instance_data.rand_file == 1)
      {
      /* choose random file */
      set_random_session_file(&instance_data);
      /* load session file */
      load_session_data(&instance_data);
      /* check config */
      check_session_data(&instance_data);
      }

    /* allocate a socket */
    cl_socket = socket(AF_INET,SOCK_STREAM, 0);
    if (cl_socket < 0)
      tj_error(&instance_data, "Failed to allocate socket: ", strerror(errno));

    /* bind to different source IP address */
    if (strcmp(instance_data.interface, "") != 0)
      {
      /* set alias */
      sprintf(bind_if_name, "%s:%d", instance_data.interface,
          instance_data.child_num + instance_data.offset - 1);

      if (instance_data.rand_ip == 1)
        {
        /* set random IP */
        sprintf(new_src_ip, "%d.%d.%d.%d", rand() % 224, rand() % 224, rand()
            % 224, rand() % 224);
        }
      else
        {
        /* set specific IP */
        sprintf(new_src_ip, instance_data.src_ip);
        }

      /* set ip on alias */
      set_ip(bind_if_name, new_src_ip);

      /* bind socket to IP address */
      check_address(&bind_data, new_src_ip);
      memcpy(&ba.sin_addr.s_addr, bind_data.server_addr,
          bind_data.server_addr_len);
      if (bind(cl_socket, (struct sockaddr *) &ba, sizeof(struct sockaddr_in))
          < 0)
        tj_error(&instance_data, "Failed to bind socket: ", strerror(errno));
      }

    /* connect */
    rc = tj_connect(cl_socket, sa, &instance_data);

    /* if no error, start transfers */
    if (rc == 0)
      {
      if (instance_data.use_ssl == 1)
        {
        /* Assign socket to SSL structure */
        SSL_set_fd(ssl, cl_socket);

        /* SSL Handshake */
        err = SSL_connect(ssl);
        if ((err) == -1)
          tj_error(&instance_data, "SSL connect ", ERR_error_string(
              ERR_get_error(), NULL));
        }
      send_data:

      buf_id = 0;
      r_timeout = 0;
      /* get next send buffer from config */
      while (buf_id < instance_data.sect_idx)
        {
        /* see what client has to say on start */
        if (buf_id == 0 && instance_data.wait_conn == 1)
          r_timeout = read_reply(cl_socket, ssl, &instance_data);
        if (r_timeout > 0)
          {
          tj_error(&instance_data, "Reply timeout, closing connection", "");
          goto finish;
          }

        /* prepare a temp send buffer */
        memset(send_buf, 0, sizeof(send_buf));
        send_buf_len = instance_data.sections_len[buf_id];
        memcpy(send_buf, instance_data.sections[buf_id], send_buf_len);

        /* process repeat tags */
        send_buf_len = repeat_data(send_buf, &instance_data, send_buf_len);

        /* process instance tags */
        send_buf_len
            = instance_num_data(send_buf, &instance_data, send_buf_len);

        /* process system tags */
        send_buf_len = system_data(send_buf, &instance_data, send_buf_len);

        /* process fuzz tags */
        send_buf_len = fuzz_data(send_buf, &instance_data, send_buf_len);

        /* process char tags */
        send_buf_len = char_data(send_buf, &instance_data, send_buf_len);

        /* send data*/
        send_data(cl_socket, ssl, &instance_data, send_buf, send_buf_len);

        /* see what server has to say, if we need to */
        if (!(instance_data.no_wait_last == 1 && buf_id
            == instance_data.sect_idx - 1))
          r_timeout = read_reply(cl_socket, ssl, &instance_data);
        if (r_timeout > 0)
          {
          tj_error(&instance_data, "Reply timeout, closing connection", "");
          goto finish;
          }

        buf_id++;
        }

      /* if keep alive send again */
      if (keepalive_counter > 1 && r_timeout == 0)
        {
        keepalive_counter--;
        goto send_data;
        }

      /* complete */
      finish:

      /* close socket */
      tj_disconnect(cl_socket, &instance_data, ssl, ctx);

      if (instance_data.debug == 1)
        goto complete;
      }
    else
      {
      /* connection failed */
      tj_error(&instance_data, "Failed to connect: ", strerror(errno));
      tj_disconnect(cl_socket, &instance_data, ssl, ctx);
      }

    /* wait if need to */
    if (instance_data.wait > 0)
      usleep(instance_data.wait);

    /* stop if only one instance debug mode */
    if (instance_data.debug == 1)
      break;

    } /* enf of main loop */
  complete:

  if (instance_data.use_ssl == 1)
    {
    err = SSL_shutdown(ssl);
    if ((err) == -1)
      tj_error(&instance_data, "SSL Disconnect ", ERR_error_string(
          ERR_get_error(), NULL));
    SSL_free(ssl);
    SSL_CTX_free(ctx);
    }
  if (instance_data.gui == 0)
    exit(EXIT_SUCCESS);
  else
    return;

  }

