/*
 *  tcpjunk - TCP protocols testing tool
 *  Contact: oferkv@gmail.com
 *
 *  tcpjunk is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2, or (at your option) any
 *  later version.
 *
 *  tcpjunk is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <openssl/crypto.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#define VER "2.649"
#define SOCK_BUF_LEN 4096
#define SESSION_MAX_LEN 1024000
#define RDELAY 2
#define SDELAY 100
#define GUI_RDELAY "2"
#define GUI_SDELAY "100"
#define BACKLOG 10
#define MAX_SECTS 100
#define SERVER_CERT "server.crt"
#define SERVER_KEY "server.key"
#define SECT_START "<section>"
#define SECT_END "</section>"
#define MAXFILES 1000

enum fuzzmode
  {
  r_none,
  r_digit,
  r_alnum,
  r_alpha,
  r_cntrl,
  r_print,
  r_special,
  r_punct,
  r_space,
  r_any,
  r_random
  };

struct tcpjunk_data
  {
    char *server_addr;
    int server_addr_len;
    int server_port;
    char session_data[SESSION_MAX_LEN];
    int session_data_len;
    int debug;
    int wait;
    int itimeout;
    int stimeout;
    int keepalive;
    int loop;
    int new_sock;
    char *sections[MAX_SECTS];
    int sections_len[MAX_SECTS];
    int sect_idx;
    int wait_conn;
    int no_wait_last;
    int tcp_opt;
    int use_ssl;
    char server_cert[32];
    char server_key[32];
    int gui;
    char session_file_name[FILENAME_MAX];
    char start_section[32];
    char end_section[32];
    int child_num;
    char interface[32];
    int offset;
    int rand_ip;
    char src_ip[32];
    int segment_size;
    int segment_delay;
    int verbose;
    int rand_file;
    char rand_file_list[MAXFILES][256];
    int rand_files_num;
  };

void junk(struct tcpjunk_data children_args);
void sjunk(struct tcpjunk_data child_data);
int
    tj_connect(int sock, struct sockaddr_in sa, struct tcpjunk_data *children_args);
void tj_disconnect(int sock,  struct tcpjunk_data *instance_data, SSL * ssl, SSL_CTX * ctx);
int read_reply(int sock, SSL *ssl, struct tcpjunk_data *children_args);
void send_data(int sock, SSL *ssl, struct tcpjunk_data *children_args,
    char *send_buf, int send_buf_len);
char fuzz_char(int ftype);
void sigchld_handler(int s);
void gui_init(int argc, char *argv[]);
int check_address(struct tcpjunk_data *instance_data, char *srv_addr);
void load_session_data(struct tcpjunk_data *instance_data);
int check_port(struct tcpjunk_data *instance_data);
void load_rand_file_list(struct tcpjunk_data *instance_data);
void set_random_session_file(struct tcpjunk_data *instance_data);
void check_session_data(struct tcpjunk_data *instance_data);
void gui_show_traffic(char *buf, int buf_len, int color);
void gui_add_log(char *buf);
void tj_error(struct tcpjunk_data *instance_data, char *message, char *param);
void set_ip(char *ifname, char *s_ip);
int fuzz_data(char *send_buf, struct tcpjunk_data *instance_data, int send_buf_len);
int system_data(char *send_buf, struct tcpjunk_data *instance_dataren_args, int send_buf_len);
void show_help();
int repeat_data(char *send_buf, struct tcpjunk_data *instance_data, int send_buf_len);
int instance_num_data(char *send_buf, struct tcpjunk_data *instance_data, int send_buf_len);
int char_data(char *send_buf, struct tcpjunk_data *instance_data, int send_buf_len);
int get_fuzz_type(char *fuzz_type, struct tcpjunk_data *instance_data);
void gui_accept();

